# Generated by roxygen2: do not edit by hand

export(GO_analysis_surprisal_analysis)
export(runSurprisalApp)
export(surprisal_analysis)
import(patchwork)
importFrom(DT,dataTableOutput)
importFrom(DT,renderDataTable)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(matlib,inv)
importFrom(shiny,HTML)
importFrom(shiny,actionButton)
importFrom(shiny,br)
importFrom(shiny,checkboxInput)
importFrom(shiny,column)
importFrom(shiny,div)
importFrom(shiny,downloadButton)
importFrom(shiny,downloadHandler)
importFrom(shiny,em)
importFrom(shiny,fileInput)
importFrom(shiny,fluidPage)
importFrom(shiny,fluidRow)
importFrom(shiny,icon)
importFrom(shiny,mainPanel)
importFrom(shiny,navbarPage)
importFrom(shiny,observeEvent)
importFrom(shiny,p)
importFrom(shiny,plotOutput)
importFrom(shiny,renderPlot)
importFrom(shiny,req)
importFrom(shiny,selectInput)
importFrom(shiny,sidebarLayout)
importFrom(shiny,sidebarPanel)
importFrom(shiny,sliderInput)
importFrom(shiny,strong)
importFrom(shiny,tabPanel)
importFrom(shiny,tags)
importFrom(shiny,updateSliderInput)
importFrom(shiny,updateTabsetPanel)
importFrom(shinycssloaders,withSpinner)
importFrom(shinyjs,disable)
importFrom(shinyjs,enable)
importFrom(shinyjs,useShinyjs)
importFrom(shinythemes,shinytheme)
importFrom(stats,quantile)
importFrom(utils,head)
